from typing import Dict
from .participant import Participant
from .helpers import find_node

class Roster:
    """
    A Battlerite's roster.
    """

    def __init__(self, data: Dict, response: Dict) -> None:
        attributes = data.get('attributes', {})
        self.id = data.get('id')
        self.shard_id = attributes.get('shardId')
        self.score = attributes.get('stats', {}).get('score')
        self.won = attributes.get('won') == 'true'

        participants = data.get('relationships', {}).get('participants', {}).get('data', [])
        self.participants = [Participant(find_node(response, p['id']), response)
                             for p in participants]



from typing import Dict

class Round:
    """
    A Battlerite's round.
    """

    def __init__(self, data: Dict) -> None:
        attributes = data['attributes']
        self.id = data['id']
        self.duration = attributes['duration']
        self.ordinal = attributes['ordinal']
        self.winning_team = attributes['stats']['winningTeam']



from .constants import ACTIONS, SUCCESS_CODES
from .match import Match
from .player import Player
from .team import Team


class Response:
    """
    This class represents a response from Battlerite's API.
    """

    def __init__(self, action, response) -> None:
        self.raw = response
        self.action = action
        self.success = response.status_code in SUCCESS_CODES
        self.rate_exceeded = response.status_code == 429
        self.rate_limit_limit = response.headers.get('X-RateLimit-Limit')
        self.rate_limit_remaining = response.headers.get('X-RateLimit-Remaining')
        self.rate_limit_reset =  response.headers.get('X-RateLimit-Reset')

    def parse(self):
        """
        Parses the result and creates objects for it.
        """
        if not self.success:
            return None

        if self.action == ACTIONS.MATCHES:
            complete = self.raw.json()
            return [Match(data, complete) for data in complete['data']]
        elif self.action == ACTIONS.PLAYERS:
            complete = self.raw.json()
            return [Player(data) for data in complete['data']]
        elif self.action == ACTIONS.TEAMS:
            complete = self.raw.json()
            return [Team(data, complete) for data in complete['data']]
        else:
            raise NotImplementedError()



from typing import Dict
from .helpers import find_node
from .round import Round

class Player:
    """
    A Battlerite Player.
    """

    def __init__(self, data: Dict) -> None:

        attributes = data.get('attributes')
        links = data.get('links', {})

        self.id = data.get('id')
        self.name = attributes.get('name')
        self.patch_version = attributes.get('patchVersion')
        self.shard_id = attributes.get('shardId')
        self.stats = attributes.get('stats')
        self.title_id = attributes.get('titleId')
        self.link_schema = links.get('schema')
        self.link = links.get('self')



from typing import Dict
from .helpers import find_node
from .player import Player

class Participant:
    """
    A Battlerite's participant.
    """

    def __init__(self, data: Dict, response: Dict) -> None:
        attributes = data.get('attributes', {})
        stats = attributes.get('stats', {})
        relationships = data.get('relationships', {})

        self.id = data.get('id')
        self.actor = attributes.get('actor')
        self.shard_id = attributes.get('shardId')
        self.ability_uses = stats.get('abilityUses')
        self.attachment = stats.get('attachment')
        self.damage_done = stats.get('damageDone')
        self.damage_received = stats.get('damageReceived')
        self.deaths = stats.get('deaths')
        self.disables_done = stats.get('disablesDone')
        self.disables_received = stats.get('disablesReceived')
        self.emote = stats.get('emote')
        self.energy_gained = stats.get('energyGained')
        self.energy_used = stats.get('energyUsed')
        self.healing_done = stats.get('healingDone')
        self.healing_received = stats.get('healingReceived')
        self.kills = stats.get('kills')
        self.mount = stats.get('mount')
        self.outfit = stats.get('outfit')
        self.score = stats.get('score')
        self.side = stats.get('side')
        self.time_alive = stats.get('timeAlive')

        player = relationships.get('player', {}).get('data', [])
        self.player = Player(find_node(response, player['id']))




"""
Contains all the helper functions that are not tied to a specific module.
"""
from typing import Dict

def find_node(data: [Dict], id_value: str) -> Dict:
    """
    Find a specific node from a response.
    If not found, KeyError is raised.
    """
    for node in data['included']:
        if node.get('id', None) == id_value:
            return node
    raise KeyError(id_value)



from typing import Dict
from .helpers import find_node
from .round import Round
from .roster import Roster

class Match:
    """
    A Battlerite's match.
    """

    def __init__(self, data: Dict, response: Dict) -> None:
        attributes = data.get('attributes', {})
        relationships = data.get('relationships', {})
        tags = attributes.get('tags')
        links = data.get('links', {})
        stats = attributes.get('stats', {})

        self.id = data.get('id')
        self.created_at = attributes.get('createdAt')
        self.duration = attributes.get('duration')
        self.game_mode = attributes.get('gameMode')
        self.patch_version = attributes.get('patchVersion')
        self.shard_id = attributes.get('shardId')
        self.map_id = stats.get('mapID')
        self.type = stats.get('type')
        self.tags = attributes.get('tags')
        self.title_id = attributes.get('titleId')
        self.link_schema = links.get('schema')
        self.link = links.get('self')
        if type(tags) is dict:
            self.ranking_type = tags.get('rankingType')
            self.server_type = tags.get('serverType')
        else:
            self.ranking_type = None
            self.server_type = None

        rounds = relationships.get('rounds', {}).get('data', [])
        rosters = relationships.get('rosters', {}).get('data', [])
        self.rounds = [Round(find_node(response, r['id'])) for r in rounds]
        self.rosters = [Roster(find_node(response, r['id']), response)
                        for r in rosters]
        self.spectators = relationships.get('spectators', [])
        self.links = [link['self'] for link in relationships.get('links', [])]



from setuptools import setup

setup(name='battlerite-client',
      version='0.1.2',
      description='Battlerite REST API client',
      url='https://github.com/DrPandemic/battlerite-client',
      author='DrPandemic',
      license='MIT',
      packages=['battlerite_client'],
      install_requires=[
          'requests',
      ],
      zip_safe=False)



import requests
from typing import Dict
from .response import Response
from .constants import ACTIONS, BASE_URL
from .team import Team
from .match import Match


class Client:
    """
    This class is used to interact with Battlerite's API.
    """

    def __init__(self, api_key: str) -> None:
        self.api_key = api_key

    def matches(self, params: Dict = {}) -> Response:
        """
        Returns matches.
        """
        return self.call(ACTIONS.MATCHES, params)

    def teams(self, season: int, player_ids: [int]) -> Response:
        """
        Returns a list of teams for given players during a season.
        """
        return self.call(ACTIONS.TEAMS,
                         {'tag[season]': season, 'tag[playerIds]': player_ids})

    def get_url(self, action: ACTIONS) -> str:
        """
        Returns the URL for a given action.
        """
        if action == ACTIONS.MATCHES:
            return f"{BASE_URL}/matches"
        elif action == ACTIONS.PLAYERS:
            return f"{BASE_URL}/players"
        elif action == ACTIONS.TEAMS:
            return f"{BASE_URL}/teams"
        else:
            raise NotImplementedError()

    def call(self, action: ACTIONS, params: Dict) -> Response:
        """
        Performs the API call and returns a Response.
        """
        headers = {
            'Accept': 'application/vnd.api+json',
            'Accept-Encoding': 'gzip',
            'Authorization': f"Bearer {self.api_key}"
        }
        url = self.get_url(action)

        if action in [ACTIONS.MATCHES, ACTIONS.TEAMS, ACTIONS.PLAYERS]:
            return Response(action,
                            requests.get(url, headers=headers, params=params))
        else:
            raise NotImplementedError()



from typing import Dict

class Team:
    """
    A Battlerite's team.
    """

    def __init__(self, data: Dict, response: Dict) -> None:
        attributes = data.get('attributes', {})
        stats = attributes.get('stats', {})
        self.id = data.get('id')
        self.shard_id = data.get('shardId')
        self.avatar = stats.get('avatar')
        self.division = stats.get('division')
        self.division_rating = stats.get('divisionRating')
        self.league = stats.get('league')
        self.losses = stats.get('losses')
        self.members = stats.get('members', [])
        self.placement_gamesLeft = stats.get('placementGamesLeft')
        self.top_division = stats.get('topDivision')
        self.top_divisionRating = stats.get('topDivisionRating')
        self.top_league = stats.get('topLeague')
        self.wins = stats.get('wins')
        self.title_id = data.get('titleId')